import QtQuick 2.5
import QtQuick.Controls 2.3
import com.zuragon.ViCANdo 1.0

Rectangle {
    width: 800
    height: 600

    color: "white"

    function debug(obj) {
        var output = '';
        for (var property in obj) {
            output += property + ': ' + obj[property] + '\n';
        }
        project.log(output); 
    }

    property var dbc: project.dbc[0]  // Use the first database file in the project. In this exmaple, it's vicando_test.dbc file

    Label {
        id: title

        text: "DBC Viewer Example"
        font.family: "Helvetica"
        anchors.topMargin: 15
        anchors.horizontalCenter: parent.horizontalCenter
        font.pointSize: 18
    }

    Label {
        id: nodeListLabel

        anchors.left: parent.left
        anchors.top: parent.top

        anchors.margins: 50

        text: "Nodes in DBC:"
    }

    Component {
        id: nodeListDelegate

        Item {
            width: 200
            height: 40

            Text {
                anchors.verticalCenter: parent.verticalCenter
                anchors.left: parent.left

                text: model.modelData.name
            }

            MouseArea {
                anchors.fill: parent

                onClicked: {
                    nodeList.currentIndex = index;
                }
            }
        }
    }

    ListView {
        id: nodeList

        property var currentNode: dbc.nodes[currentIndex]

        anchors.left: nodeListLabel.left
        anchors.top: nodeListLabel.bottom

        anchors.topMargin: 25

        width: 200
        height: 300

        interactive: false

        model: dbc.nodes

        delegate: nodeListDelegate

        highlight: Rectangle { color: "lightsteelblue"; radius: 5 }
        focus: true
    }

    Label {
        id: messageListLabel

        anchors.left: nodeList.right
        anchors.bottom: nodeListLabel.bottom

        anchors.leftMargin: 100

        text: "Messages in node:"
    }

    Component {
        id: messageListDelegate

        Item {
            width: 200
            height: 40

            Text {
                anchors.verticalCenter: parent.verticalCenter
                anchors.left: parent.left

                text: model.modelData.id + ": " + model.modelData.name
            }

            MouseArea {
                anchors.fill: parent

                onClicked: {
                    messageList.currentIndex = index;
                }
            }
        }
    }

    ListView {
        id: messageList

        property var currentMessage: messages[currentIndex]

        anchors.left: messageListLabel.left
        anchors.top: messageListLabel.bottom

        anchors.topMargin: 25

        width: 200
        height: 300

        interactive: false

        property var messages: dbc.getMessagesBySender(nodeList.currentNode)
        model: messages !== null ? messages : []

        delegate: messageListDelegate

        highlight: Rectangle { color: "lightsteelblue"; radius: 5 }
        focus: true
    }

    Label {
        id: signalListLabel

        anchors.left: messageList.right
        anchors.bottom: messageListLabel.bottom

        anchors.leftMargin: 100

        text: "Signals in message:"
    }

    Component {
        id: signalListDelegate

        Item {
            width: 200
            height: 40

            Text {
                anchors.verticalCenter: parent.verticalCenter
                anchors.left: parent.left

                text: model.modelData.name
            }

            MouseArea {
                anchors.fill: parent

                onClicked: {
                    signalList.currentIndex = index;
                }
            }
        }
    }

    ListView {
        id: signalList

        property var currentSignal: signals !== null ? signals[currentIndex] : null

        anchors.left: signalListLabel.left
        anchors.top: signalListLabel.bottom

        anchors.topMargin: 25

        width: 200
        height: 300

        interactive: false

        property var signals: messageList.currentMessage !== undefined ? messageList.currentMessage.signals : null
        model: signals !== null ? signals : []

        delegate: signalListDelegate

        highlight: Rectangle { color: "lightsteelblue"; radius: 5 }
        focus: true
    }

    Label {
        id: signalLabel

        anchors.left: signalList.right
        anchors.bottom: signalListLabel.bottom

        anchors.leftMargin: 100

        text: "Current signal: " + signalList.currentSignal !== null ? signalList.currentSignal.name : ""

        visible: signalList.currentSignal !== null
    }

    Text {
        id: signalValue

        anchors.left: signalLabel.left
        anchors.top: signalLabel.bottom

        anchors.topMargin: 25

        width: 200
        height: 50

        text: {
            if (signalList.currentSignal !== null) {
                return "Value: " + signalList.currentSignal.value + " " + signalList.currentSignal.unit;
            } else {
                return "";
            }
        }

        visible: signalList.currentSignal !== null
    }

    Text {
        id: signalInfo

        anchors.left: signalLabel.left
        anchors.top: signalValue.bottom

        width: 200
        height: 300

        text: {
            if (signalList.currentSignal === null) {
                return "";
            }

            var sig = signalList.currentSignal;

            var txt = "Name: " + sig.name + "\n";
            txt += "Combined name: " + sig.combinedName + "\n";
            txt += "Start bit: " + sig.startBit + "\n";
            txt += "Length:" + sig.length + "\n";
            txt += "Factor: " + sig.factor + "\n";
            txt += "Offset: " + sig.offset + "\n";
            txt += "Minimum: " + sig.minimum + "\n";
            txt += "Maximum: " + sig.maximum + "\n";
            if (sig.hasUnit) {
                txt += "Unit: " + sig.unit + "\n";
            }

            return txt;
        }

        visible: signalList.currentSignal !== null
    }

    SourceEventListener {
        source: project.source[0]
        dbc: project.dbc[0]

        // These different ways of providing the filter all work:
        //filter: ["MESSAGE1_1", "MESSAGE2_1"]

        //filter: [project.dbc[0].getMessageById(257)]
        //filter: [project.dbc[0].getMessageByName("MESSAGE2_2"), project.dbc[0].getMessageByName("MESSAGE3_1")]
        //filter: project.dbc[0].messages

        //filter: [dbc.getMessageById(513)]
        //filter: [dbc.getMessageByName("MESSAGE2_2"), dbc.getMessageByName("MESSAGE3_1")]

        filter: [messageList.currentMessage]
        onFilterChanged: currentMessageInfo.text = ""

        onMessageReceived: {
            var txt = "Listening for " + messageList.currentMessage.name + ":\n\n";
            if (message === messageList.currentMessage) {
                message.signals.forEach(function(sig) {
                    txt += sig.name + ": " + sig.value + " " + sig.unit + "\n";
                });
            }
            currentMessageInfo.text = txt;
        }
    }

    Text {
        id: currentMessageInfo

        anchors.left: messageListLabel.left
        anchors.top: messageList.bottom

        anchors.topMargin: 50
    }
}
